@@ -6,11 +6,11 @@
 """Module for Model class for modelling sessions data."""
 
 from collections import defaultdict
-from typing import List, Union, Dict
+from typing import Dict, List, Union
 
-from .utils.data_structures import Cmd
-from .utils import cmds_only, cmds_params_only, cmds_params_values, probabilities
 from ...common.exceptions import MsticpyException
+from .utils import cmds_only, cmds_params_only, cmds_params_values, probabilities
+from .utils.data_structures import Cmd
 
 
 # pylint: disable=too-many-instance-attributes
@@ -61,20 +61,20 @@ class Model:
         """
         if not isinstance(sessions, list):
             raise MsticpyException("`sessions` should be a list")
-        if len(sessions) == 0:
+        if not sessions:
             raise MsticpyException("`sessions` should not be an empty list")
         for i, ses in enumerate(sessions):
             if not isinstance(ses, list):
                 raise MsticpyException("each session in `sessions` should be a list")
             if len(ses) == 0:
                 raise MsticpyException(
-                    "session at index {} of `sessions` is empty. Each session "
-                    "should contain at least one command".format(i)
+                    f"session at index {i} of `sessions` is empty. Each session "
+                    "should contain at least one command"
                 )
 
-        self.start_token = "##START##"
-        self.end_token = "##END##"
-        self.unk_token = "##UNK##"
+        self.start_token = "##START##"  # nosec B105
+        self.end_token = "##END##"  # nosec B105
+        self.unk_token = "##UNK##"  # nosec B105
 
         self.sessions = sessions
         self.session_type = None
@@ -105,16 +105,16 @@ class Model:
         self.value_probs = None
         self.value_cond_param_probs = None
 
-        self.set_params_cond_cmd_probs = dict()  # type: Dict[str, Dict[str, float]]
+        self.set_params_cond_cmd_probs: Dict[str, Dict[str, float]] = {}
 
         self.session_likelihoods = None
         self.session_geomean_likelihoods = None
 
-        self.rare_windows = dict()  # type: Dict[int, list]
-        self.rare_window_likelihoods = dict()  # type: Dict[int, list]
+        self.rare_windows: Dict[int, list] = {}
+        self.rare_window_likelihoods: Dict[int, list] = {}
 
-        self.rare_windows_geo = dict()  # type: Dict[int, list]
-        self.rare_window_likelihoods_geo = dict()  # type: Dict[int, list]
+        self.rare_windows_geo: Dict[int, list] = {}
+        self.rare_window_likelihoods_geo: Dict[int, list] = {}
 
     def train(self):
         """
@@ -406,7 +406,7 @@ class Model:
                     key = set()
                     for par in pars:
                         if par in intersection_pars:
-                            key.add("{} --- {}".format(par, params[par]))
+                            key.add(f"{par} --- {params[par]}")
                         else:
                             key.add(par)
                     prob = cmds_params_values.compute_prob_setofparams_given_cmd(
@@ -507,11 +507,10 @@ class Model:
         """
         if self.session_likelihoods is None:
             self.compute_likelihoods_of_sessions()
-        result = []
-        for i in range(len(self.sessions)):
-            lik = self.session_likelihoods[i]
-            k = len(self.sessions[i])
-            result.append(lik ** (1 / k))
+        result = [
+            self.session_likelihoods[idx] ** (1 / len(session))
+            for idx, session in enumerate(self.sessions)
+        ]
 
         self.session_geomean_likelihoods = result
 
