@@ -5,12 +5,14 @@
 # --------------------------------------------------------------------------
 """Miscellaneous helper methods for Jupyter Notebooks."""
 import contextlib
+import sys
+import traceback
 from typing import List, Tuple, Union
 
 from IPython.display import display
 
-from .utility import is_ipython
 from .._version import VERSION
+from .utility import is_ipython
 
 __version__ = VERSION
 __author__ = "Ian Hellen"
@@ -73,7 +75,7 @@ class MsticpyUserError(MsticpyException):
             will be used to create a list of references in addition to
             the primary `help_uri`
         display : bool, optional
-            Display the exception when created. By default, True
+            Display the exception when created. By default, False
 
         Notes
         -----
@@ -97,9 +99,9 @@ class MsticpyUserError(MsticpyException):
             Union[str, Tuple[str, str], Tuple[Tuple[str, str], str]]
         ] = []
         title = kwargs.pop("title", "we've hit an error while running")
-        disp_exception = kwargs.pop("display", True)
+        disp_exception = kwargs.pop("display", False)
+        self._has_displayed = False
         self._output.append((f"{self.__class__.__name__} - {title}", "title"))
-
         self._output.extend(args)
 
         self._output.append("\nFor more help on fixing this error see:")
@@ -112,10 +114,13 @@ class MsticpyUserError(MsticpyException):
         ]
         if help_args:
             self._output.append("You can find other related help here:")
-            for uri in help_args:
-                self._output.append((uri, "uri"))
+            self._output.extend((uri, "uri") for uri in help_args)
+        self._context = self._format_exception_context(
+            stack=traceback.format_stack(limit=5),
+            frame_locals=sys._getframe(1).f_locals,
+        )
         if _get_config("msticpy.FriendlyExceptions") and disp_exception:
-            self._display_exception()
+            self.display_exception()
 
         # add the extra elements to the the exception standard args.
         ex_args = [title, *args, help_uri, *help_args]
@@ -134,13 +139,15 @@ class MsticpyUserError(MsticpyException):
         """Get the default help URI."""
         return self.DEF_HELP_URI
 
-    def _display_exception(self):
-        if not self._display_exceptions:
+    def display_exception(self):
+        """Output the exception HTML or text friendly exception."""
+        if not self._display_exceptions or self._has_displayed:
             return
-        if is_ipython():
+        if is_ipython(notebook=True):
             display(self)
         else:
             self._display_txt_exception()
+        self._has_displayed = True
 
     def _repr_html_(self):
         """Return HTML-formatted exception text."""
@@ -149,6 +156,7 @@ class MsticpyUserError(MsticpyException):
             div.solid {border: thin solid black; padding:10px;}
             p.title {background-color:Tomato; padding:5px;}
             ul.circle {list-style-type: circle;}
+            div.indent {text-indent: 20px; padding: 0px}
         </style>
         """
         div_tmplt = "<div class='solid'>{content}</div>"
@@ -172,6 +180,16 @@ class MsticpyUserError(MsticpyException):
                 text_line = line.replace("\n", "<br>")
                 content.append(f"{text_line}<br>")
 
+        if self._context:
+            context = [f"<div class='indent'>{line}</div>" for line in self._context]
+            content.extend(
+                (
+                    "<summary>Additional context<details>",
+                    *context,
+                    "</details></summary>",
+                )
+            )
+
         return "".join((ex_style, div_tmplt.format(content="".join(content))))
 
     def _display_txt_exception(self):
@@ -179,16 +197,16 @@ class MsticpyUserError(MsticpyException):
         print(self._get_exception_text())
 
     def _get_exception_text(self) -> str:
-        out_lines = []
+        out_lines: List[str] = []
         for line in self._output:
             if isinstance(line, tuple):
                 l_content, l_type = line
                 if isinstance(l_content, tuple):
                     l_content = l_content[0]
                 if l_type == "title":
-                    out_lines.append("-" * len(l_content))
-                    out_lines.append(l_content)
-                    out_lines.append("-" * len(l_content))
+                    out_lines.extend(
+                        ("-" * len(l_content), l_content, "-" * len(l_content))
+                    )
                 elif l_type == "uri":
                     if isinstance(l_content, tuple):
                         out_lines.append(f" - {': '.join(l_content)}")
@@ -196,8 +214,27 @@ class MsticpyUserError(MsticpyException):
                         out_lines.append(f" - {l_content}")
             else:
                 out_lines.append(line)
+        if self._context:
+            out_lines.extend(["", "Exception context:", *self._context])
         return "\n".join(out_lines)
 
+    @staticmethod
+    def _format_exception_context(stack, frame_locals):
+        context_lines: List[str] = ["Stack:", *stack, "---", "Locals:"]
+        context_lines.extend(
+            f"{var} ({type(val).__name__}) = {val}" for var, val in frame_locals.items()
+        )
+        ex_type, ex_value, ex_traceback = sys.exc_info()
+        if ex_traceback:
+            context_lines.extend(
+                (
+                    "---",
+                    "Exception was raised by:",
+                    *(traceback.format_exception(ex_type, ex_value, ex_traceback)),
+                )
+            )
+        return context_lines
+
 
 class MsticpyUserConfigError(MsticpyUserError):
     """Configuration user exception class for msticpy."""
@@ -401,6 +438,48 @@ class MsticpyImportExtraError(MsticpyUserError, ImportError):
         super().__init__(*add_args, help_uri=uri, **kwargs)
 
 
+class MsticpyMissingDependencyError(MsticpyUserError, ImportError):
+    """Exception class for Imports that are not installed."""
+
+    DEF_HELP_URI = (
+        "Installing msticpy",
+        "https://msticpy.readthedocs.io/en/latest/getting_started/Installing.html",
+    )
+
+    def __init__(
+        self, *args, help_uri: Union[Tuple[str, str], str, None] = None, **kwargs
+    ):
+        """
+        Create import missing extra exception.
+
+        Parameters
+        ----------
+        help_uri : Union[Tuple[str, str], str, None], optional
+            Override the default help URI.
+        packages : Union[str, List[str]]
+            The name of the packages or list of packages that need(s)
+            to be installed.
+
+        """
+        packages = kwargs.pop("packages", None)
+        if not packages:
+            raise AttributeError("Keyword argument 'packages' must be supplied")
+        packages = [packages] if isinstance(packages, str) else packages
+        mssg = "".join(
+            [
+                "This feature requires one or more additional packages",
+                " to be installed.\n",
+                "To do this run the command:\n",
+                f"pip install {' '.join(packages)}\n",
+                "In a notebook run:\n",
+                f"%pip install {' '.join(packages)}",
+            ]
+        )
+        add_args = [*args, mssg]
+        uri = help_uri or self.DEF_HELP_URI
+        super().__init__(*add_args, help_uri=uri, **kwargs)
+
+
 class MsticpyAzureConnectionError(MsticpyUserError):
     """Exception class for Azure Connection errors."""
 
