@@ -5,18 +5,20 @@
 # --------------------------------------------------------------------------
 """Module for Log Analytics-related configuration."""
 
-import os
+
+import contextlib
 import json
-from typing import Dict, Any, Optional
+import os
 from pathlib import Path
+from typing import Any, Dict, Optional
 
-from IPython.display import display
 import ipywidgets as widgets
+from IPython.display import display
 
+from .._version import VERSION
+from . import pkg_config
 from .exceptions import MsticpyUserConfigError
 from .utility import export, is_valid_uuid, md, md_warn
-from . import pkg_config
-from .._version import VERSION
 
 __version__ = VERSION
 __author__ = "Ian Hellen"
@@ -121,12 +123,169 @@ class WorkspaceConfig:
         self._config: Dict[str, str] = {}
         self._interactive = interactive
         self._config_file = config_file
+        self.workspace_key = workspace or "Default"
         # If config file specified, use that
         if config_file:
             self._config.update(self._read_config_values(config_file))
         else:
             self._determine_config_source(workspace)
 
+    def __getitem__(self, key: str):
+        """Allow property get using dictionary key syntax."""
+        if key in self._config:
+            return self._config[key]
+        raise KeyError
+
+    def __setitem__(self, key: str, value: Any):
+        """Allow property set using dictionary key syntax."""
+        self._config[key] = value
+
+    def __contains__(self, key: str):
+        """Allow property in test."""
+        # In operator overload
+        return key == "Type" or key in self._config or key in self.__dict__
+
+    def __repr__(self):
+        """Return contents of current config."""
+        return self._config.__repr__()
+
+    def get(self, key, default: Any = None):
+        """Return key value or default."""
+        try:
+            return self[key]
+        except KeyError:
+            return default
+
+    @property
+    def config_loaded(self) -> bool:
+        """
+        Return True if workspace id and tenant id have values.
+
+        Returns
+        -------
+        bool
+            True if configuration loaded.
+
+        """
+        ws_value = self._config.get(self.CONF_WS_ID_KEY, None)
+        ten_value = self._config.get(self.CONF_TENANT_ID_KEY, None)
+        return is_valid_uuid(ws_value) and is_valid_uuid(ten_value)  # type: ignore
+
+    @property
+    def code_connect_str(self) -> str:
+        """
+        Return the Log Analytics connection string for dev code auth.
+
+        Returns
+        -------
+        str
+            Connection string
+
+        """
+        ten_id = self._config.get(self.CONF_TENANT_ID_KEY, None)
+        ws_id = self._config.get(self.CONF_WS_ID_KEY, None)
+        if not ten_id:
+            raise KeyError(
+                f"Configuration setting for {self.CONF_TENANT_ID_KEY} "
+                + "could not be found."
+            )
+        if not ws_id:
+            raise KeyError(
+                f"Configuration setting for {self.CONF_WS_ID_KEY} "
+                + "could not be found."
+            )
+        return f"loganalytics://code().tenant('{ten_id}').workspace('{ws_id}')"
+
+    @property
+    def mp_settings(self):
+        """Return the equivalent MSTICPY settings dictionary."""
+        return {
+            self.PKG_CONF_NAME_KEY: self._config.get(self.CONF_WS_NAME_KEY),
+            self.PKG_CONF_SUB_KEY: self._config.get(self.CONF_SUB_ID_KEY),
+            self.PKG_CONF_WS_KEY: self._config.get(self.CONF_WS_ID_KEY),
+            self.PKG_CONF_TENANT_KEY: self._config.get(self.CONF_TENANT_ID_KEY),
+            self.PKG_CONF_RES_GROUP_KEY: self._config.get(self.CONF_RES_GROUP_KEY),
+        }
+
+    @classmethod
+    def from_settings(cls, settings: Dict[str, Any]):
+        """Create a WorkstationConfig from MSTICPY Workspace settings."""
+        ws_config = cls(workspace="**DUMMY_WORKSPACE**")  # type: ignore
+        ws_config._config = {  # type: ignore
+            cls.CONF_WS_NAME_KEY: settings.get(cls.PKG_CONF_NAME_KEY),  # type: ignore
+            cls.CONF_SUB_ID_KEY: settings.get(cls.PKG_CONF_SUB_KEY),  # type: ignore
+            cls.CONF_WS_ID_KEY: settings.get(cls.PKG_CONF_WS_KEY),  # type: ignore
+            cls.CONF_TENANT_ID_KEY: settings.get(cls.PKG_CONF_TENANT_KEY),  # type: ignore
+            cls.CONF_RES_GROUP_KEY: settings.get(cls.PKG_CONF_RES_GROUP_KEY),  # type: ignore
+        }
+        return ws_config
+
+    @staticmethod
+    def _read_config_values(file_path: str) -> Dict[str, str]:
+        """Read configuration file."""
+        if not file_path:
+            return {}
+        with contextlib.suppress(json.JSONDecodeError):
+            with open(file_path, "r", encoding="utf-8") as json_file:
+                if json_file:
+                    return json.load(json_file)
+        return {}
+
+    @classmethod
+    def list_workspaces(cls) -> Dict:
+        """
+        Return list of available workspaces.
+
+        Returns
+        -------
+        Dict
+            Dictionary of workspaces with workspace and tenantIds.
+
+        """
+        ws_settings = pkg_config.settings.get("AzureSentinel", {}).get("Workspaces")
+        return (
+            {
+                ws_name: {
+                    cls.PKG_CONF_WS_KEY: ws.get(cls.PKG_CONF_WS_KEY),
+                    cls.PKG_CONF_TENANT_KEY: ws.get(cls.PKG_CONF_TENANT_KEY),
+                }
+                for ws_name, ws in ws_settings.items()
+            }
+            if ws_settings
+            else {}
+        )
+
+    def prompt_for_ws(self):
+        """Display an interactive prompt for Workspace details."""
+        md_warn("No Microsoft Sentinel configuration found.")
+        md(
+            "Please enter the workspace ID and tenant Id"
+            + " to allow connection to the Microsoft Sentinel workspace."
+        )
+        ws_id_wgt = widgets.Text(description="Workspace Id:", **WIDGET_DEFAULTS)
+        ten_id_wgt = widgets.Text(description="Tenant Id:", **WIDGET_DEFAULTS)
+
+        def update_ws(chg):
+            self["workspace_id"] = chg.get("new")
+
+        def update_tnt(chg):
+            self["tenant_id"] = chg.get("new")
+
+        ws_id_wgt.observe(update_ws, names="value")
+        ten_id_wgt.observe(update_tnt, names="value")
+        display(widgets.VBox([ws_id_wgt, ten_id_wgt]))
+        md(
+            (
+                "You can avoid this prompt in future by following the"
+                " guidance in the"
+                " <a href='https://msticpy.readthedocs.io/en/latest/"
+                "data_acquisition/DataProviders.html#"
+                "connecting-to-an-azure-sentinel-workspace' "
+                "target='_blank' rel='noopener noreferrer'>"
+                "Connecting to a Microsoft Sentinel Workspace</a>"
+            ),
+        )
+
     def _determine_config_source(self, workspace):
         # First, try default MSTICPy config
         self._read_pkg_config_values(workspace_name=workspace)
@@ -168,131 +327,6 @@ class WorkspaceConfig:
                 )
             )
 
-    def __getitem__(self, key: str):
-        """Allow property get using dictionary key syntax."""
-        if key in self._config:
-            return self._config[key]
-        raise KeyError
-
-    def __setitem__(self, key: str, value: Any):
-        """Allow property set using dictionary key syntax."""
-        self._config[key] = value
-
-    def __contains__(self, key: str):
-        """Allow property in test."""
-        # In operator overload
-        return key == "Type" or key in self._config or key in self.__dict__
-
-    def __repr__(self):
-        """Return contents of current config."""
-        return self._config.__repr__()
-
-    @property
-    def config_loaded(self) -> bool:
-        """
-        Return True if workspace id and tenant id have values.
-
-        Returns
-        -------
-        bool
-            True if configuration loaded.
-
-        """
-        ws_value = self._config.get(self.CONF_WS_ID_KEY, None)
-        ten_value = self._config.get(self.CONF_TENANT_ID_KEY, None)
-        return is_valid_uuid(ws_value) and is_valid_uuid(ten_value)  # type: ignore
-
-    @property
-    def code_connect_str(self) -> str:
-        """
-        Return the Log Analytics connection string for dev code auth.
-
-        Returns
-        -------
-        str
-            Connection string
-
-        """
-        ten_id = self._config.get(self.CONF_TENANT_ID_KEY, None)
-        ws_id = self._config.get(self.CONF_WS_ID_KEY, None)
-        if not ten_id:
-            raise KeyError(
-                f"Configuration setting for {self.CONF_TENANT_ID_KEY} "
-                + "could not be found."
-            )
-        if not ws_id:
-            raise KeyError(
-                f"Configuration setting for {self.CONF_WS_ID_KEY} "
-                + "could not be found."
-            )
-        return f"loganalytics://code().tenant('{ten_id}').workspace('{ws_id}')"
-
-    @staticmethod
-    def _read_config_values(file_path: str) -> Dict[str, str]:
-        """Read configuration file."""
-        if not file_path:
-            return {}
-        try:
-            with open(file_path, "r", encoding="utf-8") as json_file:
-                if json_file:
-                    return json.load(json_file)
-        except json.JSONDecodeError:
-            pass
-        return {}
-
-    @classmethod
-    def list_workspaces(cls) -> Dict:
-        """
-        Return list of available workspaces.
-
-        Returns
-        -------
-        Dict
-            Dictionary of workspaces with workspace and tenantIds.
-
-        """
-        ws_settings = pkg_config.settings.get("AzureSentinel", {}).get("Workspaces")
-        if not ws_settings:
-            return {}
-        return {
-            ws_name: {
-                cls.PKG_CONF_WS_KEY: ws.get(cls.PKG_CONF_WS_KEY),
-                cls.PKG_CONF_TENANT_KEY: ws.get(cls.PKG_CONF_TENANT_KEY),
-            }
-            for ws_name, ws in ws_settings.items()
-        }
-
-    def prompt_for_ws(self):
-        """Display an interactive prompt for Workspace details."""
-        md_warn("No Microsoft Sentinel configuration found.")
-        md(
-            "Please enter the workspace ID and tenant Id"
-            + " to allow connection to the Microsoft Sentinel workspace."
-        )
-        ws_id_wgt = widgets.Text(description="Workspace Id:", **WIDGET_DEFAULTS)
-        ten_id_wgt = widgets.Text(description="Tenant Id:", **WIDGET_DEFAULTS)
-
-        def update_ws(chg):
-            self["workspace_id"] = chg.get("new")
-
-        def update_tnt(chg):
-            self["tenant_id"] = chg.get("new")
-
-        ws_id_wgt.observe(update_ws, names="value")
-        ten_id_wgt.observe(update_tnt, names="value")
-        display(widgets.VBox([ws_id_wgt, ten_id_wgt]))
-        md(
-            (
-                "You can avoid this prompt in future by following the"
-                " guidance in the"
-                " <a href='https://msticpy.readthedocs.io/en/latest/"
-                "data_acquisition/DataProviders.html#"
-                "connecting-to-an-azure-sentinel-workspace' "
-                "target='_blank' rel='noopener noreferrer'>"
-                "Connecting to a Microsoft Sentinel Workspace</a>"
-            ),
-        )
-
     def _read_pkg_config_values(self, workspace_name: str = None):
         as_settings = pkg_config.settings.get("AzureSentinel")
         if not as_settings:
@@ -304,6 +338,8 @@ class WorkspaceConfig:
             selected_workspace = ws_settings[workspace_name]
         elif "Default" in ws_settings:
             selected_workspace = ws_settings["Default"]
+        elif len(ws_settings) == 1:
+            selected_workspace = next(iter(ws_settings.values()))
         else:
             return {}
         if (
