@@ -6,12 +6,13 @@
 """Component edit utility functions."""
 from typing import Any, Dict, Optional, Tuple, Union
 
-import ipywidgets as widgets
 import httpx
+import ipywidgets as widgets
 
-from ..common.azure_auth_core import AzureCloudConfig
-from .comp_edit import SettingsControl
 from .._version import VERSION
+from ..auth.azure_auth_core import AzureCloudConfig
+from ..common.utility import mp_ua_header
+from .comp_edit import SettingsControl
 
 __version__ = VERSION
 __author__ = "Ian Hellen"
@@ -178,7 +179,9 @@ def get_def_tenant_id(sub_id: str) -> Optional[str]:
     get_tenant_url = (
         f"{res_mgmt_uri}/subscriptions/{{subscriptionid}}" + "?api-version=2015-01-01"
     )
-    resp = httpx.get(get_tenant_url.format(subscriptionid=sub_id))
+    resp = httpx.get(
+        get_tenant_url.format(subscriptionid=sub_id), headers=mp_ua_header()
+    )
     # Tenant ID is returned in the WWW-Authenticate header/Bearer authorization_uri
     www_header = resp.headers.get("WWW-Authenticate")
     if not www_header:
@@ -248,12 +251,14 @@ def dict_to_txt(dict_val: Union[str, Dict[str, Any]]) -> str:
     return ""
 
 
+# pylint: disable=too-many-branches
 # flake8: noqa: F821
 def get_wgt_ctrl(
     setting_path: str,
     var_name: str,
     mp_controls: "MpConfigControls",  # type: ignore
     wgt_style: Optional[Dict[str, Any]] = None,
+    instance_name: str = None,
 ) -> widgets.Widget:
     """
     Return widget appropriate to value type of `var_name`.
@@ -274,6 +279,9 @@ def get_wgt_ctrl(
                 "style": {"description_width": "100px"},
                 "layout": widgets.Layout(width="50%")
             }
+    instance_name : Optional[str]
+        An optional sub-path to an instance of this provider
+        (e.g. 'Cluster1' of path.Kusto-Cluster1)
 
     Returns
     -------
@@ -283,9 +291,18 @@ def get_wgt_ctrl(
     """
     if wgt_style is None:
         wgt_style = {}
-    var_path = f"{setting_path}.{var_name}"
+    if instance_name:
+        # insert instance name into path
+        path_elems = setting_path.split(".")
+        var_root = ".".join(
+            [path_elems[0], f"{path_elems[1]}-{instance_name}", *path_elems[2:]]
+        )
+    else:
+        var_root = setting_path
+    var_path = f"{var_root}.{var_name}"
     ctrl = mp_controls.get_control(var_path)
-    comp_defn = mp_controls.get_defn(var_path)
+    defn_path = f"{setting_path}.{var_name}"
+    comp_defn = mp_controls.get_defn(defn_path)
     if comp_defn and not isinstance(comp_defn, tuple):
         # definition is a literal
         def_value = comp_defn
