@@ -5,16 +5,16 @@
 # --------------------------------------------------------------------------
 """Module docstring."""
 from abc import ABC
-from typing import Optional
+from typing import List, Optional
 
 import ipywidgets as widgets
 
 from .._version import VERSION
 from .ce_common import (
     ITEM_LIST_LAYOUT,
-    get_wgt_ctrl,
-    get_or_create_mpc_section,
     get_defn_or_default,
+    get_or_create_mpc_section,
+    get_wgt_ctrl,
 )
 from .comp_edit import CEItemsBase, CompEditDisplayMixin
 from .compound_ctrls import get_arg_ctrl
@@ -97,7 +97,7 @@ class CEProviders(CEItemsBase, ABC):
         self.prov_options = widgets.Dropdown(
             options=self.mp_controls.get_defn(path=self._COMP_PATH).keys(),
             description="Add prov",
-            value=self.select_item.label,
+            # value=self.select_item.label,
             style=ITEM_LIST_LAYOUT["style"],
         )
         self.items_frame.children = [*(self.items_frame.children), self.prov_options]
@@ -124,15 +124,30 @@ class CEProviders(CEItemsBase, ABC):
         """Return current provider control name."""
         return self._prov_name
 
+    @property
+    def _select_prov_instance_name(self):
+        """Return the provider instance name (minus instance suffix)."""
+        return ""
+
     def _get_select_opts(self):
         """Get provider options to populate select list."""
         provs = self.mp_controls.get_value(self._COMP_PATH)
         self.prov_settings_map = _get_map(provs)
         return [(val, idx) for idx, val in enumerate(sorted(provs.keys()))]
 
-    def _populate_edit_ctrls(self, control_name: Optional[str] = None):
+    def _populate_edit_ctrls(
+        self,
+        control_name: Optional[str] = None,
+        new_provider: bool = False,
+    ):
+        """Retrieve and populate form controls for the provider to display."""
         self.edit_ctrls = _get_prov_ctrls(
-            control_name or self._prov_ctrl_name, self.mp_controls, self._COMP_PATH
+            prov_name=control_name or self._prov_ctrl_name,
+            mp_controls=self.mp_controls,
+            conf_path=self._COMP_PATH,
+            prov_instance_name=self._select_prov_instance_name
+            if not new_provider
+            else "",
         )
         self.edit_frame.children = [self.edit_ctrls]
 
@@ -153,7 +168,9 @@ class CEProviders(CEItemsBase, ABC):
         if not self.prov_options.label:
             self.set_status("Error: please select a provider name to add.")
             return
-        self._populate_edit_ctrls(control_name=self.prov_options.label)
+        self._populate_edit_ctrls(
+            control_name=self.prov_options.label, new_provider=True
+        )
         self.mp_controls.save_ctrl_values(
             f"{self._COMP_PATH}.{self.prov_options.label}"
         )
@@ -183,29 +200,47 @@ class CEProviders(CEItemsBase, ABC):
             self.set_status(status)
 
 
-def _get_prov_ctrls(prov_name, mp_controls, conf_path):
-    ctrls = []
+def _get_prov_ctrls(prov_name, mp_controls, conf_path, prov_instance_name: str = None):
+    ctrls: List[widgets.Widget] = []
     if not prov_name:
         return widgets.VBox(ctrls, layout=CompEditDisplayMixin.no_border_layout("95%"))
-    prov_path = f"{conf_path}.{prov_name}"
-    prov_defn = mp_controls.get_defn(prov_path)
+    # prov_path = f"{conf_path}.{prov_name}"
+    if prov_instance_name:
+        instance_path = f"{conf_path}.{prov_name}-{prov_instance_name}"
+    else:
+        instance_path = f"{conf_path}.{prov_name}"
+    defn_path = f"{conf_path}.{prov_name}"
+    prov_defn = mp_controls.get_defn(defn_path)
 
     for setting in prov_defn:
         if setting != "Args":
-            wgt = get_wgt_ctrl(prov_path, setting, mp_controls)
+            wgt = get_wgt_ctrl(
+                setting_path=defn_path,
+                var_name=setting,
+                mp_controls=mp_controls,
+                instance_name=prov_instance_name,
+            )
             if setting == "Provider":
                 wgt.disabled = True
             ctrls.append(wgt)
             continue
 
-        setting_path = f"{prov_path}.{setting}"
+        setting_path = f"{instance_path}.{setting}"
+        setting_defn_path = f"{defn_path}.{setting}"
         for var_name in prov_defn.get(setting):
-            comp_defn = mp_controls.get_defn(f"{setting_path}.{var_name}")
+            comp_defn = mp_controls.get_defn(f"{setting_defn_path}.{var_name}")
             if get_defn_or_default(comp_defn)[0] == "cred_key":
                 arg_ctrl = get_arg_ctrl(setting_path, var_name, mp_controls)
                 ctrls.append(arg_ctrl.hbox)
             else:
-                ctrls.append(get_wgt_ctrl(setting_path, var_name, mp_controls))
+                ctrls.append(
+                    get_wgt_ctrl(
+                        setting_path=setting_defn_path,
+                        var_name=var_name,
+                        mp_controls=mp_controls,
+                        instance_name=prov_instance_name,
+                    )
+                )
 
     return widgets.VBox(ctrls)
 
