@@ -10,21 +10,22 @@ import ipywidgets as widgets
 from IPython.display import display
 
 from .._version import VERSION
+from .ce_azure import CEAzure
 from .ce_azure_sentinel import CEAzureSentinel
 from .ce_data_providers import CEDataProviders
 from .ce_keyvault import CEKeyVault
-from .ce_azure import CEAzure
 from .ce_other_providers import CEOtherProviders
 from .ce_ti_providers import CETIProviders
 from .ce_user_defaults import CEAutoLoadComps, CEAutoLoadQProvs
-from .comp_edit import CompEditDisplayMixin, CompEditTabs, CETabControlDef
-from .mp_config_file import MpConfigFile
+from .comp_edit import CETabControlDef, CompEditDisplayMixin, CompEditTabs
 from .mp_config_control import MpConfigControls, get_mpconfig_definitions
+from .mp_config_file import MpConfigFile
 
 __version__ = VERSION
 __author__ = "Ian Hellen"
 
 
+# pylint: disable=too-many-instance-attributes
 class MpConfigEdit(CompEditDisplayMixin):
     """Msticpy Configuration helper class."""
 
@@ -69,18 +70,16 @@ class MpConfigEdit(CompEditDisplayMixin):
         self._lbl_loading = widgets.Label(value="Loading. Please wait.")
         display(self._lbl_loading)
         if isinstance(settings, MpConfigFile):
-            self.mp_conf_file = MpConfigFile(settings=settings.settings)
-            if not self.mp_conf_file.current_file and conf_filepath:
-                self.mp_conf_file.current_file = conf_filepath
+            self.mp_conf_file = MpConfigFile(
+                settings=settings.settings, file=conf_filepath
+            )
         elif isinstance(settings, dict):
-            self.mp_conf_file = MpConfigFile(settings=settings)
-            if not self.mp_conf_file.current_file and conf_filepath:
-                self.mp_conf_file.current_file = conf_filepath
+            self.mp_conf_file = MpConfigFile(settings=settings, file=conf_filepath)
         elif isinstance(settings, str):
-            self.mp_conf_file = MpConfigFile()
-            self.mp_conf_file.load_from_file(file=settings)
+            self.mp_conf_file = MpConfigFile(file=settings)
         else:
-            self.mp_conf_file = MpConfigFile()
+            # This is the default if neither settings nor conf_filepath are passed.
+            self.mp_conf_file = MpConfigFile(file=conf_filepath)
             self.mp_conf_file.load_default()
         self.tool_buttons: Dict[str, widgets.Widget] = {}
         self._inc_loading_label()
@@ -110,10 +109,13 @@ class MpConfigEdit(CompEditDisplayMixin):
         )
         self.btn_validate.on_click(self._validate_config)
         self.cb_backup = widgets.Checkbox(description="Create backup", value=False)
+        self.cb_refresh = widgets.Checkbox(description="Refresh on save", value=True)
         vbox = widgets.VBox(
             [
                 self.txt_current_file,
-                widgets.HBox([self.btn_save, self.cb_backup, self.btn_validate]),
+                widgets.HBox(
+                    [self.btn_save, self.cb_refresh, self.cb_backup, self.btn_validate]
+                ),
                 self.mp_conf_file.viewer,
             ]
         )
@@ -143,6 +145,8 @@ class MpConfigEdit(CompEditDisplayMixin):
             self.mp_conf_file.save_to_file(
                 self.txt_current_file.value, backup=self.cb_backup.value
             )
+        if self.cb_refresh.value:
+            self.mp_conf_file.refresh_mp_config()
 
     def _validate_config(self, btn):
         del btn
