@@ -4,14 +4,17 @@
 # license information.
 # --------------------------------------------------------------------------
 """MDATP OData Driver class."""
-from typing import Union, Any
+from typing import Any, Union
+
 import pandas as pd
 
-from .odata_driver import OData, QuerySource
-from ..query_defns import DataEnvironment, ensure_df_datetimes
-from ...common.azure_auth import AzureCloudConfig
-from ...common.utility import export
 from ..._version import VERSION
+from ...auth.azure_auth import AzureCloudConfig
+from ...auth.cloud_mappings import get_defender_endpoint, get_m365d_endpoint
+from ...common.data_utils import ensure_df_datetimes
+from ...common.utility import export
+from ..core.query_defns import DataEnvironment
+from .odata_driver import OData, QuerySource, _get_driver_settings
 
 __version__ = VERSION
 __author__ = "Pete Bryan"
@@ -24,7 +27,7 @@ class MDATPDriver(OData):
     CONFIG_NAME = "MicrosoftDefender"
     _ALT_CONFIG_NAMES = ["MDATPApp"]
 
-    def __init__(self, connection_str: str = None, **kwargs):
+    def __init__(self, connection_str: str = None, instance: str = "Default", **kwargs):
         """
         Instantiate MSDefenderDriver and optionally connect.
 
@@ -32,10 +35,21 @@ class MDATPDriver(OData):
         ----------
         connection_str : str, optional
             Connection string
+        instance : str, optional
+            The instance name from config to use
 
         """
         super().__init__(**kwargs)
-        api_uri, oauth_uri, api_suffix = _select_api_uris(self.data_environment)
+        cs_dict = _get_driver_settings(
+            self.CONFIG_NAME, self._ALT_CONFIG_NAMES, instance
+        )
+        self.cloud = cs_dict.pop("cloud", "global")
+        if "cloud" in kwargs and kwargs["cloud"]:
+            self.cloud = kwargs["cloud"]
+
+        api_uri, oauth_uri, api_suffix = _select_api_uris(
+            self.data_environment, self.cloud
+        )
         self.add_query_filter("data_environments", "MDE")
         self.add_query_filter("data_environments", "M365D")
         self.add_query_filter("data_environments", "MDATP")
@@ -50,6 +64,12 @@ class MDATPDriver(OData):
         self.api_root = api_uri
         self.api_ver = "api"
         self.api_suffix = api_suffix
+        if self.data_environment == DataEnvironment.M365D:
+            self.scopes = ["https://api.security.microsoft.com/AdvancedHunting.Read"]
+        else:
+            self.scopes = [
+                "https://api.securitycenter.microsoft.com/AdvancedQuery.Read"
+            ]
 
         if connection_str:
             self.current_connection = connection_str
@@ -93,18 +113,20 @@ class MDATPDriver(OData):
         return response
 
 
-def _select_api_uris(data_environment):
+def _select_api_uris(data_environment, cloud):
     """Return API and login URIs for selected provider type."""
     cloud_config = AzureCloudConfig()
     login_uri = cloud_config.endpoints.active_directory
     if data_environment == DataEnvironment.M365D:
+        base_url = get_m365d_endpoint(cloud)
         return (
-            "https://api.security.microsoft.com/",
+            base_url,
             f"{login_uri}/{{tenantId}}/oauth2/token",
             "/advancedhunting/run",
         )
+    base_url = get_defender_endpoint(cloud)
     return (
-        "https://api.securitycenter.microsoft.com/",
+        base_url,
         f"{login_uri}/{{tenantId}}/oauth2/token",
         "/advancedqueries/run",
     )
