@@ -9,8 +9,8 @@ from typing import Any, Mapping, Optional
 
 from ..._version import VERSION
 from ...common.utility import export
-from .entity import Entity
 from .account import Account
+from .entity import Entity
 from .entity_enums import ElevationToken
 from .file import File
 from .host import Host
@@ -95,7 +95,6 @@ class Process(Entity):
         self.LogonSession: Optional[HostLogonSession] = None
         super().__init__(src_entity=src_entity, **kwargs)
 
-        # pylint: disable=locally-disabled, line-too-long
         if src_event is not None:
             self._create_from_event(src_event, role)
 
@@ -132,13 +131,13 @@ class Process(Entity):
 
     @property
     def ProcessName(self) -> Optional[str]:  # noqa: N802
-        """Return the name of the process file."""
+        """Return the name of the process file."""  # noqa: N802
         file = self["ImageFile"]
         return file.Name if file else None
 
     @property
     def ProcessFilePath(self) -> Optional[str]:  # noqa: N802
-        """Return the name of the process file path."""
+        """Return the name of the process file path."""  # noqa: N802
         file = self.ImageFile
         return file.FullPath if file else None
 
