@@ -3,8 +3,13 @@
 # Licensed under the MIT License. See License.txt in the project root for
 # license information.
 # --------------------------------------------------------------------------
-"""Deprecated placeholder for eventcluster.py."""
+"""
+Deprecated - module eventcluster.py has moved.
+
+See :py:mod:`msticpy.analysis.eventcluster`
+"""
 import warnings
+
 from .._version import VERSION
 
 __version__ = VERSION
@@ -17,6 +22,7 @@ from ..analysis.eventcluster import *
 
 WARN_MSSG = (
     "This module has moved to msticpy.analysis.eventcluster\n"
-    + "Please change your import to reflect this new location."
+    "Please change your import to reflect this new location."
+    "This will be removed in MSTICPy v2.2.0"
 )
 warnings.warn(WARN_MSSG, category=DeprecationWarning)
